/************************************************************************
 *                                                                      *
 * File:        dev.h			                                *
 *                                                                      *
 * Author(s):   T. Frivold, D.Beyer.  Rooftop Communications.           *
 *                                                                      *
 * Description: Generic Device Driver API header.                       *
 *                                                                      *
 * Implementations must *NOT* make any assumptions about the numeric    *
 * values of any of the enumerations defined below.  That is, all	*
 * code should use the defined symbols only.				*
 *                                                                      *
 * Similarly, implementations must *NOT* make any assumptions about 	*
 * the size or ordering of any member in structures defined below.	*
 * That is, all code should access the members by structure		*
 * dereference only.							*
 *                                                                      *
 ************************************************************************/
#ifndef _DEV_H_
#define _DEV_H_

#include <dev_env.h>				/* env-dependencies	*/

/****************************************************************
 *
 * General definitions
 *
 ****************************************************************/

/*
 * Definitions for 'dataLen' arguments (if not set to the size of the
 * structure or buffer pointed to by data)
 */
#define DEV_API_VERSION   	0x00010301      /* version 1.3.1	*/

#define DEV_ARG_LEN_UNUSED	0xffffffff	/* data ptr is unused 	*/
#define DEV_ARG_LEN_INT32P	0xfffffffe	/* data is ptr to int32	*/

#define DEV_ARG_PTR_NULL	((void *)0)	/* unused ptr arg	*/

/*
 * Definitions for enabling/disabling signals
 */
typedef enum
{
    DevSigDisable,
    DevSigEnable,
} DevSigEnableFlag;

/****************************************************************
 *
 * Prototypes for the standard device interface functions
 *
 ****************************************************************/

/*
 * The prototypes for the standard device driver functions.  Functions
 * that match these prototypes must be filled in for the device driver
 * control block that is provided when the device is initially configured.
 *
 * The 'devHandle' value in each call is the opaque pointer registered
 * by the device driver in the control block in initialization phase.
 */

typedef	uint32	(*DevOpenFp)      (void   * devHandle, 
				   bytep    name,
				   bytep    mode);
 
typedef	uint32  (*DevCloseFp)     (void   * devHandle);
 
typedef	uint32  (*DevSigEnableFp) (void   * devHandle, 
				   uint32   sigNum, 
				   uint32   enableFlag);
 
typedef	uint32  (*DevCmdFp)       (void   * devHandle, 
				   uint32   num, 
				   uint32   qual,
				   void   * Data,
				   uint32   DataLen);
 
typedef	uint32  (*DevVarFp)       (void   * devHandle, 
				   uint32   num, 
				   uint32   qual, 
				   void   * data,
				   uint32   dataLen);
 
typedef	uint32  (*DevIdleFp)      (void   * devHandle);

typedef	uint32  (*DevReadFp)      (void   * devHandle, 
				   bytep    buffer, 
				   uint32 * len);

typedef	uint32  (*DevWriteFp)     (void   * devHandle, 
				   bytep    buffer, 
				   uint32 * len);

typedef	uint32  (*DevFlushFp)     (void  * devHandle);

/* 
 * The prototype for the function a device driver must call 
 * to report a signal to protocols.
 *
 * The actual pointer to the function will be
 * passed down when the device driver is initialized.
 */
typedef void (*DevSigFp)    (void      * protoHandle,
			     uint32      num,
			     uint32      qual,
			     void      * data,
			     uint32      dataLen,
			     uint32      retCode );

/*
 * The standard device driver control block which must be filled 
 * in by the device driver initilization function.
 *
 * The 'version' value indicates the most recent device API version that 
 * this device driver software was built to support.
 *
 * The 'type' value indicates what class of device the control block
 * holds.
 *
 * The 'devHandle' value is an opaque pointer that the device driver
 * can specify. This value will be passed back into all of the registered
 * callback functions and can be used however the device driver wishes.
 *
 * The 'numInts' value indicates the total types of signals that the
 * device driver can generate. This number must be no greater than the
 * macro ENV_MAX_NUM_DEV_SIGS. All signal numbers that the device 
 * driver signals must be >= 0 and <= 'maxSigNum'.
 */

typedef struct
{
    uint32	     version;       /* always first (among diff. versions) */
    void           * devHandle;
    uint32	     maxSigNum;
    DevOpenFp        open;
    DevCloseFp	     close;
    DevReadFp	     read;
    DevWriteFp	     write;
    DevFlushFp	     flush;
    DevSigEnableFp   sigEnable;
    DevCmdFp	     cmd;
    DevVarFp	     var;
    DevIdleFp	     idle;
} DevCntrlBlk;

/*
 * The prototype for a device driver initialization function.
 * 
 * 'version' is to identify which version of this device API the 
 * protocol software has been built for.
 *
 * The 'protoHandle' value is an opaque pointer provided by the calling
 * protocols, and should be stored for future use by the device 
 * driver as this value must be passed back when sending signals
 * to the protocols.
 *
 * The 'initData' value is an opaque pointer to data that the device
 * driver may wish to use in configuring itself.  For example, if one
 * body of code can support multiple devices of the same type, this
 * opaque data could indicate which particular device to configure.
 *
 * The 'intrpt' value is a function pointer for the device to use
 * when sending signals to the protocols.
 *
 * The 'control' value is a pointer to a DevCntrlBlk device control
 * block that must be filled in by the device driver code so that the
 * protocols can call the appropriate device driver entry points.
 */

typedef	uint32	(*DevInitFp) (uint32	      	 version,
			      void             * protoHandle,
			      void	       * initData,
			      DevSigFp		 sig,
			      DevCntrlBlk      * control);

/****************************************************************
 *
 * Standard device primitives (variables, commands, and signals)
 * and corresponding standard data structures.
 *
 ****************************************************************/

/************************
 *** GENERIC DEVICES ****
 ************************/
/*
 * Return codes
 */
typedef enum
{
    DEV_RET_START		= 0x01000000,
    DevRetOk,
    DevRetFail,
    DevRetNoInit,
    DevRetTimeOut,
    DevRetMemOut,
    DevRetHwFail,
    DevRetInvVersion,
    DevRetInvInitData,
    DevRetInvCtlBlockPtr,
    DevRetInvState,
    DevRetInvCmd,
    DevRetInvVar,
    DevRetInvSig,
    DevRetInvDev,
    DevRetInvPtr,
    DevRetInvSize,
    DevRetInvQual,
    DevRetInvParam,
    DEV_RET_NEXT,
} DevGenericRetType;

typedef enum
{
    DEV_SIG_START		= 0x02000000,
    DevSigAll,			/* used to specify all signals */
    DevSigError,
    DEV_SIG_NEXT,
} DevGenericSigType;

typedef enum
{
    DEV_VAR_START		= 0x03000000,
    DevVarVersion,
    DevVarName,
    DEV_VAR_NEXT,
} DevGenericVarType;

typedef enum
{
    DEV_CMD_START		= 0x04000000,
    DevCmdReset,
    DEV_CMD_NEXT,
} DevGenericCmdType;

/************************
 *** SERIAL DEVICES *****
 ************************/
typedef enum
{
    DEV_SER_RET_NEXT		= DEV_RET_NEXT,
} DevSerRetType;

typedef enum
{
    DevSerSigRcv		= DEV_SIG_NEXT,
    DEV_SER_SIG_NEXT,
} DevSerSigType;

typedef enum
{
    DevSerVarEolChar		= DEV_VAR_NEXT,
    DevSerVarEcho,
    DevSerVarMaxBytes,
    DevSerVarTimeout,
    DevSerVarBaud,
    DevSerVarAsciiOnly,
    DEV_SER_VAR_NEXT,
} DevSerVarType;

typedef enum
{
    DevSerCmdDrain		= DEV_CMD_NEXT,
    DEV_SER_CMD_NEXT,
} DevSerCmdType;

/************************
 *** FILE DEVICES *******
 ************************/

typedef enum
{
    DEV_FILE_RET_NEXT		= DEV_SER_RET_NEXT,
} DevFileRetType;

typedef enum
{
    DevFileSigUnused		= DEV_SER_SIG_NEXT,
    DEV_FILE_SIG_NEXT,
} DevFileSigType;

typedef enum
{
    DevFileVarSize		= DEV_SER_VAR_NEXT,
    DevFileVarOff,
    DevFileVarOSHandle,
    DevMemFileVarBase,
    DevMemFileVarLen,
    DEV_FILE_VAR_NEXT,
} DevFileVarType;

typedef enum
{
    DevFileCmdUngetChar		= DEV_SER_CMD_NEXT,
    DEV_FILE_CMD_NEXT,
} DevFileCmdType;

/************************
 *** PACKET DEVICES******
 ************************/

/*
 * Public structure used to hold packet/buffer information for:
 * 	DevPktCmdXmtPkt
 *	DevPktCmdRcvPkt
 *	DevPktSigXmtPkt
 * 	DevPktSigRcvPkt
 * 	DevPktRetPktXmtFail
 *	DevPktRetPktRcvFail
 */
typedef struct _DevPktInfo
{
    uint32	signature;
    bytep	buf;
    uint32	bufLen;
    void       *userData;
} DevPktInfo;

/*
 * Public structure used to hold MAC address/mask information for:
 * 	DevPktVarMacAdr
 */
typedef struct _DevMacAdrInfo
{
    uint32	macAdrLen;	/* Number of bytes */
    void       *macAdr;
    void       *macAdrMask;
} DevMacAdrInfo;

typedef enum
{
    DevPktRetPktRcvFail		= DEV_FILE_RET_NEXT,
    DevPktRetPktXmtFail,
    DevPktRetPktRcvError,
    DevPktRetPktXmtError,
    DEV_RET_PKT_NEXT,
} DevRetPktType;

typedef enum
{
    DevPktSigXmtPkt		= DEV_FILE_SIG_NEXT,
    DevPktSigRcvPkt,
    DEV_PKT_SIG_NEXT,
} DevPktSigType;

typedef enum
{
    DevPktVarQPkts		= DEV_FILE_VAR_NEXT,
    DevPktVarQBytes,
    DevPktVarMaxPkts,
    DevPktVarMacAdr,
    DevPktVarMaxMacAdrs,
    DevPktVarBitRate,
    DevPktVarMtu,
    DEV_PKT_VAR_NEXT,
} DevPktVarType;

typedef enum
{
    DevPktCmdXmtPkt		= DEV_FILE_CMD_NEXT,
    DevPktCmdRcvPkt,
    DEV_PKT_CMD_NEXT,
} DevPktCmdType;

/****************************************************************
 *
 * Macros for encoding the 32-bit qualifier value in the DevVarFp(),
 * DevCmdFp() and DevSigFp() function calls.
 *
 ****************************************************************/

/*
 * Each index number permits up to 256 channels or array slots.
 *
 * The basic operations are 'set' and 'get'.
 *
 * The basic input/output values are 'input', 'output', and 'both'.
 *
 * NOTE: The standard devices reserve the lower 16 bits 
 *       in the 32 bit qualifier.  The upper 16 bits are left for 
 *       device-specific extensions.
 *
 *
 */

#define	DEV_QUAL_NDX1_MASK	  0x0000007F
#define	DEV_QUAL_NDX1_SHIFT	  0
#define	DEV_QUAL_NDX1_TOTAL	  128
#define	DEV_QUAL_ISR_MASK	  0x00000080
#define	DEV_QUAL_OP_MASK	  0x00000700
#define	DEV_QUAL_IO_MASK	  0x00001800
#define	DEV_QUAL_NDX2_MASK	  0x0000E000
#define	DEV_QUAL_NDX2_SHIFT  	  13
#define	DEV_QUAL_NDX2_TOTAL	  8

/*
 * NOTE: By definition, the operation "set" and "inc" are mutually
 *       exclusive, so sharing the same bit is not a problem here.
 */
#define	DEV_QUAL_ISR		  0x00000080

#define	DEV_QUAL_OP_GET		  0x00000100
#define	DEV_QUAL_OP_SET		  0x00000200
#define	DEV_QUAL_OP_INC		  DEV_QUAL_OP_SET
#define	DEV_QUAL_OP_CLR		  0x00000400

#define	DEV_QUAL_IO_INPUT	  0x00000800
#define	DEV_QUAL_IO_RCV		  DEV_QUAL_IO_INPUT
#define	DEV_QUAL_IO_OUTPUT	  0x00001000
#define	DEV_QUAL_IO_XMT		  DEV_QUAL_IO_OUTPUT
#define	DEV_QUAL_IO_BOTH	  (DEV_QUAL_IO_INPUT | DEV_QUAL_IO_OUTPUT)

#define DEV_QUAL_USER_MASK	  0xFFFF0000

#define DEV_QUAL_INDEX1_ENCODE(i) (DEV_QUAL_NDX1_MASK & ((i) 		\
				   << DEV_QUAL_NDX1_SHIFT))
#define DEV_QUAL_INDEX1_DECODE(r) ((DEV_QUAL_NDX1_MASK & (r)) 		\
				   >> DEV_QUAL_NDX1_SHIFT)

#define DEV_QUAL_INDEX2_ENCODE(i) (DEV_QUAL_NDX2_MASK & ((i) 		\
				   << DEV_QUAL_NDX2_SHIFT))
#define DEV_QUAL_INDEX2_DECODE(r) ((DEV_QUAL_NDX2_MASK & (r)) 		\
				   >> DEV_QUAL_NDX2_SHIFT)

#endif /* _DEV_H_ */

